module vga_timing ( 

// vga controller interface
	
	reset,
	vga_clk,
	hsync,
	vsync,
	sync,
	blank,
	RGB,
	pix_val,
	read_next_pixel,
	end_of_picture
	
);

input reset;
input vga_clk;
output hsync;
output vsync;
output sync;
output blank;
output [7:0] RGB;
input [7:0] pix_val;
output read_next_pixel;
output end_of_picture;

wire hsync;
wire vsync;
wire sync;
wire blank;

wire [7:0] RGB;
wire read_next_pixel;
wire end_of_picture;
wire active;

reg [19:0] pix_count;
reg [10:0] line_count;
wire hblank;
wire vblank;

// 640 by 480 timing parameters

// horizontal timing parameters
parameter hsync_end = 96;
parameter hblank_begin = 144;
parameter hblank_end = 784;
parameter hline_end = 800;

// vertical timing parameters
parameter vsync_end = 2;
parameter vblank_begin = 34;
parameter vblank_end = 514;
parameter vframe_end = 525;

// these are all the syncs and blanks for the video timing
assign hsync = (pix_count < hsync_end) ? 1'b0 : 1'b1;
assign vsync = (line_count < vsync_end) ? 1'b0 : 1'b1;
assign hblank = ((pix_count > (hblank_begin-9)) && (pix_count < (hblank_end+8))) ? 1'b1 : 1'b0;
assign vblank = ((line_count > (vblank_begin-9)) && (line_count < (vblank_end+8))) ? 1'b1 : 1'b0;
assign sync = vsync & hsync;
assign blank = vblank & hblank;

// active indicates that the color value should be output, or black when not active
assign active = ((line_count > (vblank_begin-1)) && (line_count < (vblank_end)) && (pix_count > (hblank_begin - 2)) && (pix_count < (hblank_end - 1))) ? 1'b1 : 1'b0;

// read_next_pixel asserts one clock before the next pixel value is needed
assign read_next_pixel = ((line_count > (vblank_begin-1)) && (line_count < (vblank_end)) && (pix_count > (hblank_begin - 3)) && (pix_count < (hblank_end - 2))) ? 1'b1 : 1'b0;

// end_of_picture asserts when the last pixel of a frame is displayed on the monitor
assign end_of_picture = ((pix_count == hblank_end) && (line_count == vblank_end - 1)) ? 1'b1 : 1'b0;

assign RGB = (active) ? pix_val : 8'h00;		// red color value output to video DAC


// the heart of the time base are these two counters, the pixel and line counters
always @ (posedge vga_clk or posedge reset)
begin
	if ( reset == 1'b1 )
	begin
		pix_count <= 20'h0;
		line_count <= 11'h0;
	end
	else
	begin
		if(pix_count == (hline_end-1))
		begin
			pix_count <= 20'h0;
			if(line_count == (vframe_end-1))
			begin
				line_count <= 11'h0;
			end
			else
			begin
				line_count <= line_count + 1'b1;
			end
		end
		else
		begin
			pix_count <= pix_count + 1'b1;
		end
	end
end

endmodule
