/******************************************************************************
* Copyright ? 2004 Altera Corporation, San Jose, California, USA.             *
* All rights reserved. All use of this software and documentation is          *
* subject to the License Agreement located at the end of this file below.     *
*******************************************************************************
* Author - PRR/JRK                                                            *
*                                                                             *
* File: network_utilities.c                                                   *
*                                                                             *
* This file contains network utilities that work in conjunction with the LWIP *
* stack to bring up a design's IP address (using DHCP if available) and MAC   *
* address.                                                                    *
*                                                                             *
* Please refer to file readme.txt for notes on this software example.         *
******************************************************************************/
#include <stdio.h> 
#include <ctype.h>
#include <string.h>
#include "alt_lwip_dev.h"
#include "alt_types.h"
#include "sys/alt_flash.h"
#include "includes.h"
#include "io.h"
#include "user.h"

/*
 * The "adapter" storage here is declared so that when LWIP calls the 
 * get_ip_addr() routine, the adapter struct will be populated with the 
 * network device (Ethernet MAC) that LWIP is currently trying to setup an IP
 * address for. This is later used in the dhcp_timeout_task() routine to 
 * determine if DHCP was successful for the particular adapter.
 */
struct netif* adapter;

/*
 * MicroC/OS-II semaphores are used to signal when an IP address has been
 * aquired (declared in web_serer.c), and when the LCD may be opened for
 * write.
 */
extern OS_EVENT *attained_ip_address_sem;
#ifdef LCD_TRI_1602_0_NAME 
OS_EVENT *lcd_sem;

/* File handle for writing to LCD display. Defined in web_server.c */
extern FILE* lcdDevice;
#endif /* LCD_DISPLAY_NAME */

/*
 * die_with_error()
 * 
 * This routine is just called when a blocking error occurs with the example
 * design. It deletes the current task.
 */
void die_with_error(char err_msg[DIE_WITH_ERROR_BUFFER])
{
  printf("\n%s\n", err_msg);
  OSTaskDel(OS_PRIO_SELF);
  
  while(1);
}

/*
 * generate_and_store_mac_addr()
 * 
 * This routine is called when, upon program initialization, we discover
 * that there is no valid network settings (including MAC address) programmed
 * into flash memory. The user is prompted for their Nios development board's
 * serial number, and a unique MAC address within the Altera pool generated
 * based on that number.
 * 
 * It should be noted that this number, while unique, will likely differ from
 * the also unique (but now lost forever) MAC address programmed into the 
 * development board on the production line.
 * 
 * As we are erasing the entire flash sector, we'll re-program it with not
 * only the MAC address, but static IP, subnet, gateway, and "Use DHCP" 
 * sections. These fail-safe static settings are compatible with previous
 * Nios Ethernet designs, and allow the "factory-safe" design to behave 
 * as expected if the last flash sector is erased.
 */
err_t generate_and_store_mac_addr()
{
  err_t ret_code = ERR_IF;
  alt_u32 ser_num = 0;
  char serial_number[9], flash_content[32];
  alt_flash_fd* flash_handle;
  int i = 0;
  
  printf("Can't read the MAC address from your board (this probably means\n");
  printf("that your flash was erased). We will assign you a MAC address and\n");
  printf("static network settings\n\n");
  
  while(!ser_num)
  {
    printf("Please enter your 9-digit serial number. This is printed on a \n");
    printf("label under your Nios dev. board. The first 3 digits of the \n");
    printf("label are ASJ and the serial number follows this.\n -->");
    
    for(i=0; i<9; i++)
    {
      serial_number[i] = getchar();
      putchar(serial_number[i]);
      
      /* Handle backspaces.  How civilized. */
      if ((serial_number[i] == 0x08) && (i >= 0)) 
      {
        i--;
      }
    }
    printf("\n");
            
    for(i=0; i<9; i++)
    {
      if (isdigit(serial_number[i]))
      {
        ser_num *= 10;
        ser_num += serial_number[i] - '0';
      }
      else
      {
        ser_num = 0;
        printf("Serial number only contains decimal digits and is non-zero\n");
        break;
      }
    }
    
    if (ser_num)
    {
      /* This says the image is safe */
      flash_content[0] = 0xfe;
      flash_content[1] = 0x5a;
      flash_content[2] = 0x0;
      flash_content[3] = 0x0;
      
      /* This is the Altera Vendor ID */
      flash_content[4] = 0x0;
      flash_content[5] = 0x7;
      flash_content[6] = 0xed;
      
      /* Reserverd Board identifier for erase boards */
      flash_content[7] = 0xFF;
      flash_content[8] = (ser_num & 0xff00) >> 8;
      flash_content[9] = ser_num & 0xff;

      /* Then comes a 16-bit "flags" field (not used with LWIP) */
      flash_content[10] = 0xFF;
      flash_content[11] = 0xFF;
      
      /* Then comes the static IP address */
      flash_content[12] = IPADDR0;
      flash_content[13] = IPADDR1;
      flash_content[14] = IPADDR2;
      flash_content[15] = IPADDR3;
      
      /* Then comes the static nameserver address (not used with LWIP)  */
      flash_content[16] = 0xFF;
      flash_content[17] = 0xFF;
      flash_content[18] = 0xFF;
      flash_content[19] = 0xFF;
      
      /* Then comes the static subnet mask */
      flash_content[20] = MSKADDR0;
      flash_content[21] = MSKADDR1;
      flash_content[22] = MSKADDR2;
      flash_content[23] = MSKADDR3;
      
      /* Then comes the static gateway address */
      flash_content[24] = GWADDR0;
      flash_content[25] = GWADDR1;
      flash_content[26] = GWADDR2;
      flash_content[27] = GWADDR3;
      
      /* And finally whether to use DHCP - set all bits to be safe */
      flash_content[28] = 0xFF;
      flash_content[29] = 0xFF;
      flash_content[30] = 0xFF;
      flash_content[31] = 0xFF;
      
      /* Write the MAC address to flash */
      flash_handle = alt_flash_open_dev("/dev/flash");
      if (flash_handle)
      {
        alt_write_flash(flash_handle, LAST_SECTOR_OFFSET, flash_content, 32);
        alt_flash_close_dev(flash_handle);
        ret_code = ERR_OK;
      }
    }
  }
  return ret_code;    
}

/*
 * get_mac_addr()
 *
 * Read the MAC address in a board specific way. This routine must be provided
 * by the LWIP user as each board will have its MAC address stored in a 
 * different way.
 */
err_t get_mac_addr(alt_lwip_dev* lwip_dev)
{
  err_t ret_code = ERR_OK;
  alt_u32 signature;
  struct netif* netif = lwip_dev->netif;

  /*
   * Our boards have the MAC address stored in the last sector of flash. The 
   * format is in byte order: fe 5a 00 00 <6-byte MAC Address>
   */
#if defined(FLASH_NAME)      
  signature = IORD_32DIRECT(LAST_FLASH_SECTOR, 0);

  if (signature != 0x00005afe)
  {
    ret_code = generate_and_store_mac_addr();
  }
  
  if (ret_code == ERR_OK)
  {
    netif->hwaddr[0] = IORD_8DIRECT(LAST_FLASH_SECTOR, 4);
    netif->hwaddr[1] = IORD_8DIRECT(LAST_FLASH_SECTOR, 5);
    netif->hwaddr[2] = IORD_8DIRECT(LAST_FLASH_SECTOR, 6);
    netif->hwaddr[3] = IORD_8DIRECT(LAST_FLASH_SECTOR, 7);
    netif->hwaddr[4] = IORD_8DIRECT(LAST_FLASH_SECTOR, 8);
    netif->hwaddr[5] = IORD_8DIRECT(LAST_FLASH_SECTOR, 9);
    
    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n", 
            netif->hwaddr[0],
            netif->hwaddr[1],
            netif->hwaddr[2],
            netif->hwaddr[3],
            netif->hwaddr[4],
            netif->hwaddr[5]);

  }
#else
  ret_code = ERR_IF;

  perror( "Not an Altera board.\n");
  perror( "You need to modify the function get_mac_addr() \n");
  perror( "to set a MAC address for your board");
#endif /* (Altera Nios Dev board definition) */

  return ret_code;
}

/*
 * get_ip_addr()
 * 
 * This routine is called by LWIP to obtain an IP address for the specified 
 * network adapter. Like the MAC address, obtaining an IP address is very
 * system-dependant and therefore this function is exported for the developer
 * to control.
 * 
 * In our system, we are either attempting DHCP auto-negotiation of IP address,
 * or we are setting our own static IP, Gateway, and Subnet Mask addresses our
 * self. This routine is where that happens.
 */
int get_ip_addr(alt_lwip_dev* lwip_dev, struct ip_addr* ipaddr, 
                struct ip_addr* netmask, struct ip_addr* gw,
                int* use_dhcp)
{
  int ret_code = 0;


  if (!strcmp(lwip_dev->name, "/dev/" LWIP_DEFAULT_IF))
  {
#if LWIP_DHCP == 1
    *use_dhcp = 1;

    /*
     * If we are telling LWIP to attempt DHCP, all that is needed is to save 
     * the the LWIP device in question to a the global netif "adapter"; the 
     * DHCP timeout task will then querry this device to see whether an IP
     * address has been set.
     * 
     * If we are not attempting DHCP, static network settings are assigned here
     * and we go on our merry way.
     */
    adapter = lwip_dev->netif;

#ifdef LCD_TRI_1602_0_NAME  
    fprintf(lcdDevice, "Using DHCP to 1");
    fprintf(lcdDevice, " nfind IP Addr2"); 
#endif /* LCD_DISPLAY_NAME */
    printf("Using DHCP to find an IP Address\n");

#else
    IP4_ADDR(ipaddr, IPADDR0,IPADDR1,IPADDR2,IPADDR3);
    IP4_ADDR(gw, GWADDR0,GWADDR1,GWADDR2,GWADDR3);
    IP4_ADDR(netmask, MSKADDR0,MSKADDR1,MSKADDR2,MSKADDR3);
    *use_dhcp = 0;

    printf("Static IP Address is %d.%d.%d.%d\n", 
        ip4_addr1(ipaddr),
        ip4_addr2(ipaddr),
        ip4_addr3(ipaddr),
        ip4_addr4(ipaddr));

#ifdef  LCD_TRI_1602_0_NAME  

      fprintf(lcdDevice, "Static IP Addr1");
      fprintf(lcdDevice, " %d.%d.%d.%d",
        ip4_addr1(ipaddr),
        ip4_addr2(ipaddr),
        ip4_addr3(ipaddr),
        ip4_addr4(ipaddr));
#endif /* LCD_DISPLAY_NAME */

#endif /* LWIP_DHCP */



    ret_code = 1;
  }

  return ret_code;
}

/*
 * dhcp_timeout_task()
 * 
 * This LWIP-called routine is responsible for checking to see whether DHCP 
 * was sucessful. Here we check to see whether a timeout (120 seconds in this
 * case) has elapsed. If so, display an error, assign a static address, and 
 * bail out. If not, continue checking for DHCP success and bail out once
 * a DHCP address assignment is complete.
 * 
 * This routine also displays the assigned IP address to the user (via STDOUT 
 * and, if equipped, LCD screen.
 * 
 * Note: LCD_DISPLAY_NAME is defined in user.h to be the LCD interface for
 *       the "standard" example design.
 */
#if LWIP_DHCP == 1
void dhcp_timeout_task()
{
  int have_address = 0, timeout_dhcp = 0;
  struct ip_addr ipaddr, netmask, gw;
  INT8U return_code = OS_NO_ERR;
 
#ifdef LCD_TRI_1602_0_NAME  
  lcd_sem = OSSemCreate(1);
#endif /* LCD_DISPLAY_NAME */

  OSSemPend(attained_ip_address_sem, 0, &return_code);
  
#ifdef LCD_TRI_1602_0_NAME  
  OSSemPend(lcd_sem, 0, &return_code);
#endif /* LCD_DISPLAY_NAME */
  
#ifdef LCD_TRI_1602_0_NAME  
  lcdDevice = fopen(LCD_DISPLAY_NAME, "w");
#endif /* LCD_DISPLAY_NAME */

  while(1)
  {
    OSTimeDlyHMSM(0,0,0,100);

    /* 
     * For demo purposes we need to use DHCP to allocate an IP Address
     * and LWIP had no mechanism to tell us one is allocated. Wait two 
     * minutes if we don't have one we use a static IP address
     */
    if((timeout_dhcp < DHCP_TIMEOUT) && (have_address == 0))
    {
      timeout_dhcp++;
      if (!ip_addr_isany(&adapter->ip_addr))
      {
        have_address = 1;
        printf("Assigned IP Address is %d.%d.%d.%d\n\n",
                ip4_addr1(&adapter->ip_addr),
                ip4_addr2(&adapter->ip_addr),
                ip4_addr3(&adapter->ip_addr),
                ip4_addr4(&adapter->ip_addr));

#ifdef LCD_TRI_1602_0_NAME  

        fprintf(lcdDevice, "DHCP IP Addr is 1");
        fprintf(lcdDevice, " %d.%d.%d.%d2",
                ip4_addr1(&adapter->ip_addr),
                ip4_addr2(&adapter->ip_addr),
                ip4_addr3(&adapter->ip_addr),
                ip4_addr4(&adapter->ip_addr));
#endif /* LCD_DISPLAY_NAME */
      }
    }
    /*
     * You're in a bit of trouble if this happens, you could assign a static
     * IP address but how do we choose it?
     */
    if (timeout_dhcp ==  DHCP_TIMEOUT)
    {
      dhcp_stop(adapter);
      IP4_ADDR(&ipaddr, IPADDR0,IPADDR1,IPADDR2,IPADDR3);
      IP4_ADDR(&gw, GWADDR0,GWADDR1,GWADDR2,GWADDR3);
      IP4_ADDR(&netmask, MSKADDR0,MSKADDR1,MSKADDR2,MSKADDR3);
      
      netif_set_addr( adapter,&ipaddr, &netmask, &gw);
      netif_set_up(adapter);
      
      printf("DHCP Failed to assign an IP Address\n\n"); 
      printf("\nUsing static address: %d.%d.%d.%d\n\n\n",
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));

#ifdef LCD_DISPLAY_NAME  
      /* Clear the LCD screen */
      fprintf(lcdDevice, "\x1b");
      fprintf(lcdDevice, "[2J");
      fprintf(lcdDevice, "DHCP Failed using\n %d.%d.%d.%d",
        ip4_addr1(&adapter->ip_addr),
        ip4_addr2(&adapter->ip_addr),
        ip4_addr3(&adapter->ip_addr),
        ip4_addr4(&adapter->ip_addr));
#endif /* LCD_DISPLAY_NAME */
      have_address = 1;
    }
    
    if (have_address == 1)
    {
      /* 
       * We have an IP address this task has served it's purpose - kill it.
       */
#ifdef LCD_DISPLAY_NAME        
       fclose(lcdDevice);
#endif /* LCD_DISPLAY_NAME */
       OSSemPost(attained_ip_address_sem);
#ifdef LCD_DISPLAY_NAME        
       OSSemPost(lcd_sem);
#endif /* LCD_DISPLAY_NAME */
       OSTaskDel(OS_PRIO_SELF);
    }
  }
}
#endif /* LWIP_DHCP */


/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2004 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/
